
/*************************************
* This file is not in use
**************************************/

/*
 * AUDIODriverDescriptors.c
 *
 *  Created on: Feb 17, 2009
 *      Author: Espen Karlsen
 *
 */

#include "AUDIODriverDescriptors.h"
#include "AUDIOStreamingInterfaceDescriptor.h"
#include "AUDIOOutputTerminalDescriptor.h"
#include "AUDIOInputTerminalDescriptor.h"
#include "AUDIOHeaderDescriptor.h"
#include "AUDIOFormatTypeOneDescriptor.h"
#include "AUDIOFeatureUnitDescriptor.h"
#include "AUDIOEndpointDescriptor.h"
#include "AUDIODataEndpointDescriptor.h"
#include "AUDIOFeedbackEndpointDescriptor.h"
#include "AUDIOControlInterfaceDescriptor.h"
#include "AUDIODriver.h"
#include "AUDIOGenericDescriptor.h"

#include "USBGenericDescriptor.h"
#include "USBDeviceDescriptor.h"
#include "USBConfigurationDescriptor.h"
#include "USBInterfaceDescriptor.h"
#include "USBDDriverDescriptors.h"
#include "USBDStringDescriptor.h"
#include "USBEndpointDescriptor.h"
#include "doxaUSB.h"
#include "compiler.h"




/// \page "Audio Speaker Device Codes"
///
/// This page lists the %device IDs and release number of the USB Audio Speaker
/// %device.
///
/// !Codes
/// - AUDDSpeakerDriverDescriptors_VENDORID
/// - AUDDSpeakerDriverDescriptors_PRODUCTID
/// - AUDDSpeakerDriverDescriptors_RELEASE

/// Device vendor ID.
#define AUDIODriverDescriptors_VENDORID            0x03EB// Atmel
/// Device product ID.
#define AUDIODriverDescriptors_PRODUCTID           0x2861
/// Device release number in BCD format.
#define AUDIODriverDescriptors_RELEASE             0x0100


//------------------------------------------------------------------------------
//         Local types
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
/// Header descriptor with 1 interface.
//------------------------------------------------------------------------------
typedef struct {

    /// Header descriptor.
    AUDHeaderDescriptor header;
    /// Id of the first grouped interface - Speaker.
    unsigned char bInterface0;

} __attribute__ ((packed)) AUDHeaderDescriptor1; // GCC



//------------------------------------------------------------------------------
/// Feature unit descriptor with 3 channel controls (master, right, left).
//------------------------------------------------------------------------------
typedef struct {

    /// Feature unit descriptor.
    AUDFeatureUnitDescriptor feature;
    /// Available controls for each channel.
    unsigned char bmaControls[3];
    /// Index of a string descriptor for the feature unit.
    unsigned char iFeature;

} __attribute__ ((packed)) AUDFeatureUnitDescriptor3; // GCC



//------------------------------------------------------------------------------
/// List of descriptors for detailling the audio control interface of a
/// device using a USB audio speaker driver.
//------------------------------------------------------------------------------
typedef struct {

    /// Header descriptor (with one slave interface).
    AUDHeaderDescriptor1 header;
    /// Input terminal descriptor.
    AUDInputTerminalDescriptor input;
    /// Output terminal descriptor.
    AUDOutputTerminalDescriptor output;
    /// Feature unit descriptor.
    AUDFeatureUnitDescriptor3 feature;

} __attribute__ ((packed)) AUDDSpeakerDriverAudioControlDescriptors; // GCC



//------------------------------------------------------------------------------
/// Format type I descriptor with one discrete sampling frequency.
//------------------------------------------------------------------------------
typedef struct {

    /// Format type I descriptor.
    AUDFormatTypeOneDescriptor formatType;
    /// Sampling frequency in Hz.
    unsigned char tSamFreqMin[3];
   // unsigned char tSamFreqMax[3];

} __attribute__ ((packed)) AUDFormatTypeOneDescriptor1; // GCC



//------------------------------------------------------------------------------
/// Holds a list of descriptors returned as part of the configuration of
/// a USB audio speaker device.
//------------------------------------------------------------------------------
typedef struct {

    /// Standard configuration.
    USBConfigurationDescriptor configuration;
    /// Audio control interface.
    USBInterfaceDescriptor control;
    /// Descriptors for the audio control interface.
    AUDDSpeakerDriverAudioControlDescriptors controlDescriptors;
    //- AUDIO OUT
    /// Streaming out interface descriptor (with no endpoint, required).
    USBInterfaceDescriptor streamingOutNoIsochronous;							// Silent interface
    /// Streaming out interface descriptor.
    USBInterfaceDescriptor streamingOut;										// Audio streaming interface
    /// Audio class descriptor for the streaming out interface.
    AUDStreamingInterfaceDescriptor streamingOutClass;							// Audio specific interface descriptor
    /// Stream format descriptor.
    AUDFormatTypeOneDescriptor1 streamingOutFormatType;							// Audio format descriptor
    /// Streaming out endpoint descriptor.
    AUDEndpointDescriptor streamingOutEndpoint;									// Isochronous out EP standard descriptor
    /// Audio class descriptor for the streaming out endpoint.
    AUDDataEndpointDescriptor streamingOutDataEndpoint;							// Isochronous out EP audio specific descriptor
    // Audio feedback descriptor
//    AUDIOFeedbackEndpointDescriptor feedbackEndpoint;							// Feedback EP descriptor

} __attribute__ ((packed)) AUDIODriverConfigurationDescriptors; // GCC



//------------------------------------------------------------------------------
//         Local variables
//------------------------------------------------------------------------------

/// Device descriptor.
const USBDeviceDescriptor deviceDescriptor = {

    sizeof(USBDeviceDescriptor),
    USBGenericDescriptor_DEVICE,
    Swap16(USBDeviceDescriptor_USB2_00),
    AUDIODeviceDescriptor_CLASS,
    AUDIODeviceDescriptor_SUBCLASS,
    AUDIODeviceDescriptor_PROTOCOL,
    BOARD_USB_ENDPOINTS_MAXPACKETSIZE(0),
    Swap16(AUDIODriverDescriptors_VENDORID), // Atmel vendor ID
    Swap16(AUDIODriverDescriptors_PRODUCTID), // Product ID
    Swap16(AUDIODriverDescriptors_RELEASE), // Product release 0.01
    1, // Manufacturer string descriptor
    2, // Product string descriptor
    3, // Serial number string descriptor
    1 // One possible configuration
};



/// USB device qualifier descriptor.
const USBDeviceQualifierDescriptor qualifierDescriptor = {

    sizeof(USBDeviceDescriptor),
    USBGenericDescriptor_DEVICE,
    Swap16(USBDeviceDescriptor_USB2_00),
    AUDIODeviceDescriptor_CLASS,
    AUDIODeviceDescriptor_SUBCLASS,
    AUDIODeviceDescriptor_PROTOCOL,
    BOARD_USB_ENDPOINTS_MAXPACKETSIZE(0),
    1, // Device has one possible configuration
    0  // Reserved
};


/// Configuration descriptors for a USB audio speaker driver.
const AUDIODriverConfigurationDescriptors configurationDescriptors = {

    // Configuration descriptor - Not class- specific
    {
        sizeof(USBConfigurationDescriptor),
        USBGenericDescriptor_CONFIGURATION,
        Swap16(sizeof(AUDIODriverConfigurationDescriptors)),	// Total length of configuration descriptor
        2, // This configuration has 2 interfaces
        1, // This is configuration #1
        0, // No string descriptor
        BOARD_USB_BMATTRIBUTES,
        USBConfigurationDescriptor_POWER(100)
    },
    // Audio control interface standard descriptor
    {
        sizeof(USBInterfaceDescriptor),
        USBGenericDescriptor_INTERFACE,
        AUDIODriverDescriptors_CONTROL,
        0, // This is alternate setting #0
        0, // This interface uses no endpoint
        AUDControlInterfaceDescriptor_CLASS,
        AUDControlInterfaceDescriptor_SUBCLASS,
        AUDControlInterfaceDescriptor_PROTOCOL,
        0 // No string descriptor
    },
    // Audio control interface descriptors
    {
        // Header descriptor
        {
            {
                sizeof(AUDHeaderDescriptor1),
                AUDGenericDescriptor_INTERFACE,
                AUDGenericDescriptor_HEADER,
                Swap16(AUDHeaderDescriptor_AUD1_00),
                Swap16(sizeof(AUDDSpeakerDriverAudioControlDescriptors)),
                1 // One streaming interface
            },
            AUDIODriverDescriptors_STREAMING
        },
        // Input terminal descriptor
        {
            sizeof(AUDInputTerminalDescriptor),
            AUDGenericDescriptor_INTERFACE,
            AUDGenericDescriptor_INPUTTERMINAL,
            AUDIODriverDescriptors_INPUTTERMINAL,
            Swap16(AUDInputTerminalDescriptor_USBSTREAMING),
            AUDIODriverDescriptors_OUTPUTTERMINAL,
            AUDIODriver_NUMCHANNELS,
            Swap16(AUDInputTerminalDescriptor_LEFTFRONT
            | AUDInputTerminalDescriptor_RIGHTFRONT),
            0, // No string descriptor for channels
            0 // No string descriptor for input terminal
        },
        // Output terminal descriptor
        {
            sizeof(AUDOutputTerminalDescriptor),
            AUDGenericDescriptor_INTERFACE,
            AUDGenericDescriptor_OUTPUTTERMINAL,
            AUDIODriverDescriptors_OUTPUTTERMINAL,
            Swap16(AUDOutputTerminalDescriptor_SPEAKER),
            AUDIODriverDescriptors_INPUTTERMINAL,
            AUDIODriverDescriptors_FEATUREUNIT,
            0 // No string descriptor
        },
        // Feature unit descriptor
        {
            {
                sizeof(AUDFeatureUnitDescriptor3),
                AUDGenericDescriptor_INTERFACE,
                AUDGenericDescriptor_FEATUREUNIT,
                AUDIODriverDescriptors_FEATUREUNIT,
                AUDIODriverDescriptors_INPUTTERMINAL,
                1, // 1 byte per channel for controls
            },
            {
              /*  AUDFeatureUnitDescriptor_VOLUME |*/ AUDFeatureUnitDescriptor_MUTE, // Master channel controls
                0, // Right channel controls
                0  // Left channel controls
            },
            0 // No string descriptor
        }
    },

    // Audio streaming interface with 0 endpoints - zero bandwidth
    {
        sizeof(USBInterfaceDescriptor),
        USBGenericDescriptor_INTERFACE,
        AUDIODriverDescriptors_STREAMING,
        0, // This is alternate setting #0
        0, // This interface uses no endpoints
        AUDStreamingInterfaceDescriptor_CLASS,
        AUDStreamingInterfaceDescriptor_SUBCLASS,
        AUDStreamingInterfaceDescriptor_PROTOCOL,
        4 // string descriptor
    },

    // Audio streaming interface with data endpoint - normal operation
    {
        sizeof(USBInterfaceDescriptor),
        USBGenericDescriptor_INTERFACE,
        AUDIODriverDescriptors_STREAMING,
        1, // This is alternate setting #1
        1, // This interface uses 2 endpointS
        AUDStreamingInterfaceDescriptor_CLASS,
        AUDStreamingInterfaceDescriptor_SUBCLASS,
        AUDStreamingInterfaceDescriptor_PROTOCOL,
        5 // string descriptor
    },

    // Audio streaming class-specific descriptor
    {
        sizeof(AUDStreamingInterfaceDescriptor),
        AUDGenericDescriptor_INTERFACE,
        AUDStreamingInterfaceDescriptor_GENERAL,
        AUDIODriverDescriptors_INPUTTERMINAL,
        0, // No internal delay because of data path
        Swap16(AUDFormatTypeOneDescriptor_PCM)

    },
    // Format type I descriptor
    {
        {
            sizeof(AUDFormatTypeOneDescriptor1),
            AUDGenericDescriptor_INTERFACE,
            AUDStreamingInterfaceDescriptor_FORMATTYPE,
            AUDFormatTypeOneDescriptor_FORMATTYPEONE,
            AUDIODriver_NUMCHANNELS,
            AUDIODriver_BYTESPERSAMPLE,
            AUDIODriver_BYTESPERSAMPLE*8,
            1// AUDFormatTypeOneDescriptor_CONTINUOUS - Number of supported frequencies
        },
        {
        	AUDIODriver_SAMPLERATE & 0xFF,
			(AUDIODriver_SAMPLERATE >> 8) & 0xFF,
			(AUDIODriver_SAMPLERATE >> 16) & 0xFF
        },
//        {
//        	AUDIODriver_SAMPLERATE_MAX & 0xFF,
//			(AUDIODriver_SAMPLERATE_MAX >> 8) & 0xFF,
//			(AUDIODriver_SAMPLERATE_MAX >> 16) & 0xFF
//        },

    },
    // Audio streaming endpoint standard descriptor
    {
        sizeof(AUDEndpointDescriptor),
        USBGenericDescriptor_ENDPOINT,
        USBEndpointDescriptor_ADDRESS(
            USBEndpointDescriptor_OUT,
            AUDIODriverDescriptors_DATAOUT),
        USBEndpointDescriptor_ISOCHRONOUS,// | 0x01 << 2,
        Swap16(BOARD_USB_ENDPOINTS_MAXPACKETSIZE(AUDIODriverDescriptors_DATAOUT)),
        0, // Polling interval = 2^(x-1) milliseconds (1 ms)
        0, //10 // This is not a synchronization endpoint
        0 //USBEndpointDescriptor_ADDRESS(USBEndpointDescriptor_IN, AUDIODriverDescriptors_FBIN) // No associated synchronization endpoint
    },
    // Audio streaming endpoint class-specific descriptor
    {
        sizeof(AUDDataEndpointDescriptor),
        AUDGenericDescriptor_ENDPOINT,
        AUDDataEndpointDescriptor_SUBTYPE,
        0,//0x1, // No attributes
        0, // Endpoint is asynchronous synchronized
        0 // Endpoint is asynchronous synchronized
    },
//    {
//    	sizeof(AUDIOFeedbackEndpointDescriptor),
//    	USBGenericDescriptor_ENDPOINT,
//    	USBEndpointDescriptor_ADDRESS(USBEndpointDescriptor_IN, AUDIODriverDescriptors_FBIN),
//    	USBEndpointDescriptor_ISOCHRONOUS | 0x01 << 4,
//    	Swap16(BOARD_USB_ENDPOINTS_MAXPACKETSIZE(AUDIODriverDescriptors_DATAOUT)),
//    	10
//    }
};




/// String descriptor with the supported languages.
const unsigned char languageIdDescriptor[] = {
    USBStringDescriptor_LENGTH(1),
    USBGenericDescriptor_STRING,
    USBStringDescriptor_ENGLISH_US
};


/// Manufacturer name.
const unsigned char manufacturerDescriptor[] = {

    USBStringDescriptor_LENGTH(11),
    USBGenericDescriptor_STRING,
    USBStringDescriptor_UNICODE('E'),
    USBStringDescriptor_UNICODE('s'),
    USBStringDescriptor_UNICODE('p'),
    USBStringDescriptor_UNICODE('e'),
    USBStringDescriptor_UNICODE('n'),
    USBStringDescriptor_UNICODE('&'),
    USBStringDescriptor_UNICODE('M'),
    USBStringDescriptor_UNICODE('a'),
    USBStringDescriptor_UNICODE('g'),
    USBStringDescriptor_UNICODE('n'),
    USBStringDescriptor_UNICODE('e')
};

/// Product name.
const unsigned char productDescriptor[] = {

    USBStringDescriptor_LENGTH(8),
    USBGenericDescriptor_STRING,
    USBStringDescriptor_UNICODE('D'),
    USBStringDescriptor_UNICODE('O'),
    USBStringDescriptor_UNICODE('X'),
    USBStringDescriptor_UNICODE('A'),
    USBStringDescriptor_UNICODE(' '),
    USBStringDescriptor_UNICODE('D'),
    USBStringDescriptor_UNICODE('A'),
    USBStringDescriptor_UNICODE('C')
};

/// Product serial number.
const unsigned char serialNumberDescriptor[] = {

    USBStringDescriptor_LENGTH(4),
    USBGenericDescriptor_STRING,
    USBStringDescriptor_UNICODE('0'),
    USBStringDescriptor_UNICODE('0'),
    USBStringDescriptor_UNICODE('0'),
    USBStringDescriptor_UNICODE('1')
};

// Configuration description
const unsigned char audioConfigDescriptor[] = {

		USBStringDescriptor_LENGTH(5),
		USBGenericDescriptor_STRING,
		USBStringDescriptor_UNICODE('A'),
		USBStringDescriptor_UNICODE('u'),
		USBStringDescriptor_UNICODE('d'),
		USBStringDescriptor_UNICODE('i'),
		USBStringDescriptor_UNICODE('o')
};

const unsigned char interface0Descriptor[] = {

		USBStringDescriptor_LENGTH(5),
		USBGenericDescriptor_STRING,
		USBStringDescriptor_UNICODE('I'),
		USBStringDescriptor_UNICODE('n'),
		USBStringDescriptor_UNICODE('f'),
		USBStringDescriptor_UNICODE(' '),
		USBStringDescriptor_UNICODE('0')
};

const unsigned char interface1Descriptor[] = {

		USBStringDescriptor_LENGTH(5),
		USBGenericDescriptor_STRING,
		USBStringDescriptor_UNICODE('I'),
		USBStringDescriptor_UNICODE('n'),
		USBStringDescriptor_UNICODE('f'),
		USBStringDescriptor_UNICODE(' '),
		USBStringDescriptor_UNICODE('1')
};


/// Array of pointers to the four string descriptors.
const unsigned char *stringDescriptors[] = {

    languageIdDescriptor,
    manufacturerDescriptor,
    productDescriptor,
    serialNumberDescriptor,
    interface0Descriptor,
    interface1Descriptor


};


//------------------------------------------------------------------------------
//         Exported functions
//------------------------------------------------------------------------------

// List of descriptors required by an USB audio speaker device driver.
const USBDDriverDescriptors audioDriverDescriptors = {

    &deviceDescriptor,
    (const USBConfigurationDescriptor *) &configurationDescriptors,
    &qualifierDescriptor,
    (const USBConfigurationDescriptor *) &configurationDescriptors,
    &deviceDescriptor,
    (const USBConfigurationDescriptor *) &configurationDescriptors,
    &qualifierDescriptor,
    (const USBConfigurationDescriptor *) &configurationDescriptors,
    stringDescriptors,
    5 // Number of string descriptors
};






// List of descriptors used by the device.
//const USBDDriverDescriptors audioDriverDescriptors = {
//
//    &deviceDescriptor,
//    (const USBConfigurationDescriptor *) &configurationDescriptors,
//#ifdef BOARD_USB_UDPHS
//    0, // No full-speed device qualifier descriptor
//    0, // No full-speed other speed configuration descriptor
//    &deviceDescriptor,
//    (const USBConfigurationDescriptor *) &configurationDescriptors,
//    0, // No high-speed device qualifier descriptor
//    0, // No high-speed other speed configuration descriptor
//#else
//    0, // No full-speed device qualifier descriptor
//    0, // No full-speed other speed configuration descriptor
//    0, // No high-speed device descriptor
//    0, // No high-speed configuration descriptor
//    0, // No high-speed device qualifier descriptor
//    0, // No high-speed other speed configuration descriptor
//#endif
//    0, // No string descriptor
//    0  // No string descriptor
//};


