/* ----------------------------------------------------------------------------
 *         ATMEL Microcontroller Software Support
 * ----------------------------------------------------------------------------
 * Copyright (c) 2008, Atmel Corporation
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer below.
 *
 * Atmel's name may not be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ----------------------------------------------------------------------------
 */

#ifndef AUDIODRIVERDESCRIPTORS_H_
#define AUDIODRIVERDESCRIPTORS_H_


//------------------------------------------------------------------------------
//         Headers
//------------------------------------------------------------------------------

#include "USBDDriverDescriptors.h"

//------------------------------------------------------------------------------
//         Definitions
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
#define AUDIODeviceDescriptor_CLASS			0xEF
#define AUDIODeviceDescriptor_SUBCLASS		0x02	// 16-bit PCM audio data
#define AUDIODeviceDescriptor_PROTOCOL		0x01 	// Default mono
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
/// \page "Audio Endpoint Numbers"
///
/// This page lists the endpoint number settings for USB Audio Speaker device.
///
/// !Endpoints
/// - AUDDSpeakerDriverDescriptors_DATAOUT
/// - AUDDSpeakerDriverDescriptors_DATAIN

/// Data out endpoint number.
#define AUDIODriverDescriptors_DATAOUT        0x05
/// Data in endpoint number
#define AUDIODriverDescriptors_DATAIN         0x06

#define AUDIODriverDescriptors_FBIN			  0x01
#define AUDIODriverDescriptors_FBOUT		  0x02
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
/// \page "Audio Interface IDs"
///
/// This page lists the interface numbers for USB Audio Speaker device.
///
/// !Interfaces
/// - AUDDSpeakerDriverDescriptors_CONTROL
/// - AUDDSpeakerDriverDescriptors_STREAMING
/// - AUDDSpeakerDriverDescriptors_STREAMINGIN

/// Audio control interface ID.
#define AUDIODriverDescriptors_CONTROL            0
/// Audio streaming interface ID (OUT, for playback).
#define AUDIODriverDescriptors_STREAMING          1
/// Audio streaming interface ID (IN, for record).
#define AUDIODriverDescriptors_STREAMINGIN        2
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
/// \page "Audio Speaker Entity IDs"
///
/// This page lists the entity IDs for USB Audio Speaker device.
///
/// !Entities
/// - AUDDSpeakerDriverDescriptors_INPUTTERMINAL
/// - AUDDSpeakerDriverDescriptors_OUTPUTTERMINAL
/// - AUDDSpeakerDriverDescriptors_FEATUREUNIT
/// - AUDDSpeakerDriverDescriptors_INPUTTERMINAL_REC
/// - AUDDSpeakerDriverDescriptors_OUTPUTTERMINAL_REC
/// - AUDDSpeakerDriverDescriptors_FEATUREUNIT_REC

/// Playback input terminal ID.
#define AUDIODriverDescriptors_INPUTTERMINAL      0
/// Playback output terminal ID.
#define AUDIODriverDescriptors_OUTPUTTERMINAL     1
/// Playback feature unit ID.
#define AUDIODriverDescriptors_FEATUREUNIT        2
/// Record input terminal ID.
#define AUDIODriverDescriptors_INPUTTERMINAL_REC  3
/// Record output terminal ID.
#define AUDIODriverDescriptors_OUTPUTTERMINAL_REC 4
/// Record feature unit ID
#define AUDIODriverDescriptors_FEATUREUNIT_REC    5
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//         Exported variables
//------------------------------------------------------------------------------

extern const USBDDriverDescriptors audioDriverDescriptors;

#endif /* AUDIODRIVERDESCRIPTORS_H_ */
