/******************************************************************************
 * ui.h
 *
 * \brief This file defines functions to control the buttons and LEDs.
 *
 *	Four buttons are connected to the PIOB module to control volume up, volume
 *	down, mute and source select. Three leds are connected to the same module
 *	to indicate mute status and source selection.
 *
 *  Created on: May 24, 2009
 *      Author: Espen Karlsen
 ******************************************************************************/

#ifndef UI_H_
#define UI_H_

/*! \brief Setup pio controller for buttons and leds.
 *	Sets up the pio controller for leds and buttons and
 *	registers the pin change interrupt routine and the
 *	RTC interrupt routine.
 */
void init_buttons(void);

/*! \brief Transfer led status to outputs.
 *
 */
void leds_update(void);

/*! \brief Pin change interrupt handler
 * Called every time the level on a pin connected to a button
 * changes if the corresponding interrupt is enabled.
 */
void buttons_int_handler(void);

/*! \brief Real time counter interrupt handler
 * Called every time the RTC flips over
 */
void rtc_int_handler(void);

#endif /* UI_H_ */
